@@ -7,6 +7,7 @@
 
 #include "kclockformat.h"
 
+#include "settingsmodel.h"
 #include "utilmodel.h"
 
 #include <QLocale>
@@ -17,30 +18,26 @@ KclockFormat::KclockFormat(QObject *parent)
     : QObject(parent)
     , m_timer(new QTimer(this))
 {
-    connect(m_timer, &QTimer::timeout, this, &KclockFormat::updateTime);
+    connect(SettingsModel::instance(), &SettingsModel::timeFormatChanged, this, &KclockFormat::updateCurrentTime);
+    connect(m_timer, &QTimer::timeout, this, &KclockFormat::updateCurrentTime);
+
     this->startTimer();
 }
 
-void KclockFormat::updateTime()
+QString KclockFormat::currentTime()
 {
-    if (m_minutesCounter == 60) {
-        m_currentTime = QLocale::system().toString(QTime::currentTime(), UtilModel::instance()->isLocale24HourTime() ? "hh:mm" : "h:mm ap");
-        Q_EMIT timeChanged();
-        m_minutesCounter = 0;
-        m_hoursCounter++;
-    }
-    if (m_hoursCounter == 60) {
-        m_hoursCounter = 0;
-        m_hours = QTime::currentTime().hour() >= 12 ? QTime::currentTime().hour() - 12 : QTime::currentTime().hour();
-        Q_EMIT hourChanged();
-    }
-    m_minutesCounter++;
-    Q_EMIT secondChanged();
+    return m_currentTime;
+}
+
+void KclockFormat::updateCurrentTime()
+{
+    m_currentTime = QLocale::system().toString(QTime::currentTime(), UtilModel::instance()->timeFormat());
+    Q_EMIT timeChanged();
 }
 
 QString KclockFormat::formatTimeString(int hours, int minutes)
 {
-    return QLocale::system().toString(QTime(hours, minutes), QStringLiteral("hh:mm ap"));
+    return QLocale::system().toString(QTime(hours, minutes), UtilModel::instance()->timeFormat());
 }
 
 bool KclockFormat::isChecked(int dayIndex, int daysOfWeek)
@@ -57,23 +54,21 @@ bool KclockFormat::isChecked(int dayIndex, int daysOfWeek)
 
 void KclockFormat::startTimer()
 {
-    m_currentTime = QLocale::system().toString(QTime::currentTime(), UtilModel::instance()->isLocale24HourTime() ? "hh:mm" : "h:mm ap");
-    m_hours = QTime::currentTime().hour() >= 12 ? QTime::currentTime().hour() - 12 : QTime::currentTime().hour();
-    m_minutesCounter = (QTime::currentTime().msecsSinceStartOfDay() / 1000) % 60; // seconds to next minute
-    m_hoursCounter = QTime::currentTime().minute();
+    updateCurrentTime();
     m_timer->start(1000);
 }
+
 WeekModel::WeekModel(QObject *parent)
     : QAbstractListModel(parent)
 {
     // init week and flag value
     int dayFlag[] = {1, 2, 4, 8, 16, 32, 64}, i = 0;
     for (int j = QLocale::system().firstDayOfWeek(); j <= 7; j++) {
-        m_listItem[i++] = std::tuple<QString, int>(QLocale::system().dayName(j, QLocale::ShortFormat), dayFlag[j - 1]);
+        m_listItem[i++] = std::tuple<QString, int>(QLocale::system().dayName(j, QLocale::LongFormat), dayFlag[j - 1]);
     }
 
     for (int j = 1; j < QLocale::system().firstDayOfWeek(); j++) {
-        m_listItem[i++] = std::tuple<QString, int>(QLocale::system().dayName(j, QLocale::ShortFormat), dayFlag[j - 1]);
+        m_listItem[i++] = std::tuple<QString, int>(QLocale::system().dayName(j, QLocale::LongFormat), dayFlag[j - 1]);
     }
 }
 
