@@ -13,6 +13,14 @@
 #include <QThread>
 #include <QTimer>
 
+// manually disable powerdevil, even if found
+static bool noPowerDevil = false;
+
+void Utilities::disablePowerDevil(bool disable)
+{
+    noPowerDevil = disable;
+}
+
 Utilities &Utilities::instance()
 {
     static Utilities singleton;
@@ -26,17 +34,19 @@ Utilities::Utilities(QObject *parent)
                                      QStringLiteral("org.kde.Solid.PowerManagement"),
                                      QDBusConnection::sessionBus(),
                                      this))
+    , m_hasPowerDevil(false)
     , m_timer(new QTimer(this))
 {
-    connect(m_timer, &QTimer::timeout, this, [this] {
-        if (m_activeTimerCount <= 0) {
-            QApplication::exit();
-        }
-    });
+    // TODO: It'd be nice to be able to have the daemon off if no alarms/timers are running.
+    // However, with the current implementation, the client continuously thinks it is off.
+    //     connect(m_timer, &QTimer::timeout, this, [this] {
+    //         if (m_activeTimerCount <= 0) {
+    //             QApplication::exit();
+    //         }
+    //     });
 
     // if PowerDevil is present, we can rely on PowerDevil to track time, otherwise we do it ourself
-    if (m_interface->isValid()) {
-        // test Plasma 5.20 PowerDevil schedule wakeup feature
+    if (m_interface->isValid() && !noPowerDevil) {
         m_hasPowerDevil = hasWakeup();
     }
 
@@ -44,32 +54,34 @@ Utilities::Utilities(QObject *parent)
                                                                 QStringLiteral("org.kde.PowerManagement"),
                                                                 this,
                                                                 QDBusConnection::ExportScriptableSlots);
-    qDebug() << "Register Utility on DBus: " << success;
-    if (this->hasPowerDevil()) {
+    qDebug() << "Registered on DBus:" << success;
+
+    if (hasPowerDevil()) {
         qDebug() << "PowerDevil found, using it for time tracking.";
     } else {
         initWorker();
-
         qDebug() << "PowerDevil not found, using wait worker thread for time tracking.";
     }
 
-    auto m_watcher = new QDBusServiceWatcher{POWERDEVIL_SERVICE_NAME,
-                                             QDBusConnection::sessionBus(),
-                                             QDBusServiceWatcher::WatchForRegistration | QDBusServiceWatcher::WatchForUnregistration};
-    connect(m_watcher, &QDBusServiceWatcher::serviceRegistered, this, [this]() {
-        qDebug() << "powerdevil found";
-        m_hasPowerDevil = hasWakeup();
-        if (m_hasPowerDevil && m_timerThread)
-            m_timerThread->quit();
+    if (!noPowerDevil) { // do not watch for powerdevil if we explicitly turned it off
+        auto m_watcher = new QDBusServiceWatcher{POWERDEVIL_SERVICE_NAME,
+                                                 QDBusConnection::sessionBus(),
+                                                 QDBusServiceWatcher::WatchForRegistration | QDBusServiceWatcher::WatchForUnregistration};
+        connect(m_watcher, &QDBusServiceWatcher::serviceRegistered, this, [this]() {
+            qDebug() << "PowerDevil found on DBus";
+            m_hasPowerDevil = hasWakeup();
+            if (m_hasPowerDevil && m_timerThread)
+                m_timerThread->quit();
 
-        Q_EMIT needsReschedule();
-    });
-    connect(m_watcher, &QDBusServiceWatcher::serviceUnregistered, this, [this]() {
-        m_hasPowerDevil = false;
-        initWorker();
+            Q_EMIT needsReschedule();
+        });
+        connect(m_watcher, &QDBusServiceWatcher::serviceUnregistered, this, [this]() {
+            m_hasPowerDevil = false;
+            initWorker();
 
-        Q_EMIT needsReschedule();
-    });
+            Q_EMIT needsReschedule();
+        });
+    }
 
     // exit after 1 min if nothing happens
     m_timer->start(60 * 1000);
@@ -82,71 +94,74 @@ bool Utilities::hasPowerDevil()
 
 int Utilities::scheduleWakeup(quint64 timestamp)
 {
-    if (this->hasPowerDevil()) {
+    if (hasPowerDevil()) {
         QDBusReply<uint> reply = m_interface->call(QStringLiteral("scheduleWakeup"), QStringLiteral("org.kde.kclockd"), QDBusObjectPath("/Utility"), timestamp);
         if (reply.isValid()) {
-            m_cookies.append(reply.value());
+            m_powerDevilCookies.append(reply.value());
         } else {
-            qDebug() << "invalid reply, error: " << reply.error();
+            qDebug() << "Invalid reply, error: " << reply.error();
         }
         return reply.value();
     } else {
-        m_list.append({++m_cookie, timestamp});
-        this->schedule();
+        m_waitWorkerCookies.append({++m_cookie, timestamp});
+        schedule();
         return m_cookie;
     }
 }
 
 void Utilities::clearWakeup(int cookie)
 {
-    if (this->hasPowerDevil()) {
-        auto index = m_cookies.indexOf(cookie);
+    if (hasPowerDevil()) {
+        auto index = m_powerDevilCookies.indexOf(cookie);
         if (index != -1) {
             m_interface->call(QStringLiteral("clearWakeup"), cookie);
-            m_cookies.removeAt(index);
+            m_powerDevilCookies.removeAt(index);
         }
     } else {
-        auto index = 0;
-        for (const auto &tuple : std::as_const(m_list)) {
-            if (cookie == std::get<0>(tuple)) {
-                break;
+        for (auto index = m_waitWorkerCookies.begin(); index < m_waitWorkerCookies.end(); ++index) {
+            int pairCookie = (*index).first;
+            if (cookie == pairCookie) {
+                m_waitWorkerCookies.erase(index);
             }
-            ++index;
         }
-        m_list.removeAt(index);
 
-        this->schedule();
+        // ensure that we schedule the next queued wakeup
+        schedule();
     }
 }
 
 void Utilities::wakeupCallback(int cookie)
 {
-    qDebug() << "wakeup callback";
-    auto index = m_cookies.indexOf(cookie);
+    qDebug() << "Received wakeup callback.";
+    auto index = m_powerDevilCookies.indexOf(cookie);
 
     if (index == -1) {
         // something must be wrong here, return and do nothing
-        qDebug() << "callback ignored (wrong cookie)";
-        return;
+        qDebug() << "Callback ignored (wrong cookie).";
     } else {
         // remove token
-        m_cookies.removeAt(index);
+        m_powerDevilCookies.removeAt(index);
         Q_EMIT wakeup(cookie);
     }
 }
 
 void Utilities::schedule()
 {
-    auto minTime = std::numeric_limits<unsigned long long>::max();
+    auto eternity = std::numeric_limits<unsigned long long>::max();
+    auto minTime = eternity;
 
-    for (const auto &tuple : std::as_const(m_list)) {
-        if (minTime > std::get<1>(tuple)) {
-            minTime = std::get<1>(tuple);
-            m_currentCookie = std::get<0>(tuple);
+    for (auto tuple : m_waitWorkerCookies) {
+        if (minTime > tuple.second) {
+            minTime = tuple.second;
+            m_currentCookie = tuple.first;
         }
     }
-    m_worker->setNewTime(minTime); // Unix uses int64 internally for time, if we don't have anything to wait, we wait to year 2262 A.D.
+
+    if (minTime != eternity) { // only schedule worker if we have something to wait on
+        m_worker->setNewTime(minTime); // Unix uses int64 internally for time
+    }
 }
+
 void Utilities::initWorker()
 {
     if (!m_timerThread) {
@@ -155,8 +170,8 @@ void Utilities::initWorker()
         m_worker->moveToThread(m_timerThread);
         connect(m_worker, &AlarmWaitWorker::finished, this, [this] {
             // notify time is up
-            Q_EMIT this->wakeup(m_currentCookie);
-            this->clearWakeup(m_currentCookie);
+            Q_EMIT wakeup(m_currentCookie);
+            clearWakeup(m_currentCookie);
         });
     }
     m_timerThread->start();
@@ -170,11 +185,7 @@ bool Utilities::hasWakeup()
                                                     QStringLiteral("Introspect"));
     QDBusReply<QString> result = QDBusConnection::sessionBus().call(m);
 
-    if (result.isValid() && result.value().indexOf(QStringLiteral("scheduleWakeup")) >= 0) { // have this feature
-        return true;
-    } else {
-        return false;
-    }
+    return result.isValid() && result.value().indexOf(QStringLiteral("scheduleWakeup")) >= 0;
 }
 
 void Utilities::exitAfterTimeout()
